
from .records    import *

import pyaas

pyaas.argparser.add_argument('--db',
    help='Select storage engine')

pyaas.argparser.add_argument('--initdb',
    action='store_true',
    help='Initialize the database')




import pyaas
import logging


def _loadModuleImpl(moduleName, defaultPath):
    """
    Based on a module impl name, attempt to load the python module
    If the module name is fully qualified, use that string only.  Otherwise,
    Prepend the defaultPath
    :param moduleName: module impl name (google or pgsql or my.module.module)
    :param defaultPath: default path to find built in module impls
    :return: The loaded python module
    """

    if '.' in moduleName:
        moduleName = moduleName
    else:
        # all auth mechanisms need to be in this path
        moduleName = defaultPath + '.' + moduleName

    try:
        module = __import__(moduleName)
    except ImportError:
        raise pyaas.error('Unknown module: "%s"', moduleName)

    for sub in moduleName.split('.')[1:]:
        module = getattr(module, sub)

    return module

def _loadModuleClass(clsname, moduleName, defaultPath):
    """
    Load the class within the module
    :param clsname: The name of the class within the package (Login, Cache, or Database)
    :param moduleName: module impl name (google or pgsql or my.module.module)
    :param defaultPath: default path to find built in module impls
    :return: The class
    """
    module = _loadModuleImpl(moduleName, defaultPath)
    try:
        cls = getattr(module, clsname)
    except AttributeError:
        raise pyaas.error('Failed to load class %s from %s', clsname, moduleName)
    return cls


def _initializeObject(obj, **options):
    """
    Attampts to invoke an initialize function/method on the given object
    :param obj:
    :param options:
    :return:
    """
    func = None
    if hasattr(obj, 'initialize'):
        f = getattr(obj, 'initialize')
        if callable(f):
            func = f
    if func is None and hasattr(obj, 'Initialize'):
        f = getattr(obj, 'Initialize')
        if callable(f):
            func = f

    if func is not None and func.__self__ is obj:
        func(**options)


class PyaasModule(object):
    """
    Defines a pyaas module.  Pyaas modules are parts of the system that may be configured
    and/or extended.  These are things like database engines, authentication systems or
    cache backends.
    Available modules are: storage, cache, and auth
    """
    def __init__(self, name=None, defaultPackage=None, className=None):
        self._name = name
        self._defaultPackage = defaultPackage
        self._className = className
        self._cls = None
        self._obj = None
        self._implName = None
        self._options = None

    @property
    def name(self):
        return self._name

    @name.setter
    def name(self, value):
        self._name = value

    @property
    def defaultPackage(self):
        return self._defaultPackage

    @defaultPackage.setter
    def defaultPackage(self, value):
        self._defaultPackage = value

    @property
    def className(self):
        return self._className

    @className.setter
    def className(self, value):
        self._className = value

    @property
    def implName(self):
        if self._implName is None:
            name = self.name
            if name in pyaas.args.__dict__:
                self._implName = pyaas.args.__dict__[name]
            elif pyaas.config.has_option('modules', name):
                self._implName = pyaas.config.get('modules', name)
        return self._implName

    @property
    def options(self):
        if self._options is None:
            impl = self.implName
            self._options = {}
            if pyaas.config.has_section(impl):
                self._options = dict(pyaas.config.items(impl))
                logging.debug('%s options: %s', self.name, self._options)
        return self._options

    def getInstance(self):
        if self._obj is None:
            cls = self.getClass()
            impl = self.implName

            if cls is None or impl is None:
                return None

            options = self.options
            self._obj = cls(**options)
            _initializeObject(self._obj)
        return self._obj

    def getClass(self):
        if self._cls is None:
            impl = self.implName
            if impl is None:
                return None

            self._cls = _loadModuleClass(self.className, impl, self.defaultPackage)
            if self._cls is None:
                return None

            _initializeObject(self._cls, **self.options)
        return self._cls

    def load(self, application=None):
        raise Exception("load not implemented!")




class Auth(PyaasModule):
    """
    Pyaas auth module
    """
    def __init__(self):
        super(Auth, self).__init__('auth', 'pyaas.web.handlers.auth', 'Login')

    def load(self, application=None):
        login = self.getClass()
        if login:
            # extend the patterns and settings accordingly
            application.patterns.extend([
                ( r'/login',  login                      ),
                ( r'/logout', pyaas.web.handlers.auth.Logout ),
                ])

            application.settings['login_url'] = '/login'

            logging.debug('Enabled authentication: %s', self.implName)


class Storage(PyaasModule):
    """
    Pyaas database module
    """
    def __init__(self):
        super(Storage, self).__init__('storage', 'pyaas.storage.engines', 'Database')

    def load(self, application=None):
        pyaas.db = self.getInstance()
        if pyaas.db:
            logging.debug('Enabled storage: %s', self.implName)


class Cache(PyaasModule):
    """
    Pyaas cache module
    """
    def __init__(self):
        super(Cache, self).__init__('cache', 'pyaas.storage.cache', 'Cache')

    def load(self, application=None):
        pyaas.cache = self.getInstance()
        if pyaas.cache:
            logging.debug('Enabled cache: %s', self.implName)





import datetime

import pyaas

import tornado.auth

#
# Facebook Authentication
#

FBURL = '%s://%s/login/facebook?next=%s'


class Login(tornado.web.RequestHandler, tornado.auth.FacebookGraphMixin):
    @tornado.web.asynchronous
    def get(self):
        redir = tornado.escape.url_escape(self.get_argument('next', '/'))
        my_url = FBURL % (self.request.protocol, self.request.host, redir)

        code = self.get_argument('code', None)

        if code:
            self.get_authenticated_user(
                redirect_uri  = my_url,
                client_id     = pyaas.config.get('facebook', 'api_key'),
                client_secret = pyaas.config.get('facebook', 'secret'),
                code          = code,
                callback      = self._on_auth
            )
        else:
            self.authorize_redirect(
                redirect_uri = my_url,
                client_id    = pyaas.config.get('facebook', 'api_key'),
                extra_params = {'scope': 'email'}
            )

    def _on_auth(self, fbuser):
        if not fbuser:
            raise tornado.web.HTTPError(500, 'Facebook authentication failed')

        profile = pyaas.db.FindProfile('fbid', fbuser['id'])
        if not profile:
            self.facebook_request(
                '/me',
                access_token = fbuser['access_token'],
                callback     = self._on_me
            )
        else:
            self.set_secure_cookie('uid', str(profile['uid']))
            self.redirect(self.get_argument('next', '/'))

    def _on_me(self, fbuser):
        profile = pyaas.db.FindProfile('email', fbuser['email'])
        if not profile:
            profile = dict(
                email      = fbuser['email'],
                display    = fbuser['name'],
                fbid       = fbuser['id'],
                firstLogin = datetime.datetime.now()
            )

            uid = pyaas.db.SaveProfile(profile)
            self.set_secure_cookie('uid', str(uid))

        else:
            self.set_secure_cookie('uid', str(profile['uid']))
            # TODO: update facebook id

        self.redirect(self.get_argument('next', '/'))




import sys
import os
import argparse
import collections
import inspect
import logging

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

import pyaas

pyaas.argparser = argparse.ArgumentParser()

pyaas.argparser.add_argument('--address', '-a',
    help = 'Interface to bind to')

pyaas.argparser.add_argument('--port', '-p',
    type=int,
    help='Port to bind to')

pyaas.argparser.add_argument('--newcookie',
    action='store_true',
    help='Generate a new cookie')

pyaas.argparser.add_argument('--ini', '-i',
    help='Specify additional ini file')

pyaas.argparser.add_argument('--debug', '-d',
    action='store_true',
    help='Print verbose debugging information')

logging.basicConfig(
    format  = '%(asctime)s %(levelname)-8s %(message)s',
    datefmt = '%Y-%m-%d %H:%M:%S',
    level   = logging.INFO
)


def load(program=None):
    if program is None:
        program = os.path.basename(sys.argv[0])

        if program.endswith('.py'):
            program = program.rsplit('.', 1)[0]

    pyaas.args = pyaas.argparser.parse_args()

    pyaas.config = configparser.SafeConfigParser(dict_type=collections.OrderedDict)
    pyaas.config.optionxform = str

    ini_files = [
        os.path.join(pyaas.prefix, 'etc', pyaas.namespace, program + '.ini'),
        os.path.join(pyaas.prefix, 'etc', pyaas.namespace, program + '.ini.local')
    ]

    if pyaas.args.ini:
        ini_files.append(pyaas.args.ini)

    try:
        ok = pyaas.config.read(ini_files)
    except configparser.ParsingError as e:
        raise pyaas.error('Unable to parse file: %s', e)

    if not ok:
        raise pyaas.error('Unable to read config file(s): %s', ini_files)

    # setup file log
    root = logging.getLogger()
    fh = logging.FileHandler(os.path.join(pyaas.paths.var, program + '.log'))
    fh.setLevel(logging.INFO)

    if pyaas.args.debug:
        root.setLevel(logging.DEBUG)
        fh.setLevel(logging.DEBUG)

    formatter = logging.Formatter(
        fmt = '%(asctime)s %(levelname)-8s %(message)s',
        datefmt = '%Y-%m-%d %H:%M:%S',
        )

    fh.setFormatter(formatter)

    # add the handlers to the logger
    root.addHandler(fh)


def setPrefix(prefix=None):
    if not prefix:
        # inspect who called this function
        frames = inspect.getouterframes(inspect.currentframe())
        # get the caller frame
        frame = frames[-1]
        # get the filename of the caller
        prefix = os.path.abspath(frame[1])
        # get the directory name of the file
        prefix = os.path.dirname(prefix)

        if prefix.endswith(os.path.sep + 'bin'):
            prefix = os.path.join(prefix, '..')
            prefix = os.path.abspath(prefix)

    prefix = os.path.abspath(prefix)
    if pyaas.prefix != prefix:
        pyaas.prefix = prefix
        logging.info('Setting prefix to "%s"', pyaas.prefix)


def setNameSpace(namespace=None):
    if namespace is None:
        # inspect who called this function
        frames = inspect.getouterframes(inspect.currentframe())
        # get the caller frame
        frame = frames[-1]
        namespace = os.path.basename(frame[1]).split('.')[0]

    if namespace != pyaas.namespace:
        pyaas.namespace = namespace
        logging.info('Setting namespace to "%s"', pyaas.namespace)


def init(prefix='', namespace='', settings=None):
    """
    Call this guy to init pyaas stuffs
    :param prefix: The root path of the app
    :param namespace: The namespace
    :param settings: Alternative name of ini file to load
    :return: None
    """

    # Set my prefix
    setPrefix(prefix)
    # Set my namespace
    setNameSpace(namespace)

    Paths = collections.namedtuple('Paths', ['etc', 'share', 'var'])
    pyaas.paths = Paths(
        etc   = os.path.join(pyaas.prefix, 'etc',   pyaas.namespace),
        share = os.path.join(pyaas.prefix, 'share', pyaas.namespace),
        var   = os.path.join(pyaas.prefix, 'var',   pyaas.namespace),
        )

    # Init settings
    pyaas.settings.load(settings)

def postinit():
    # Init global modules
    pyaas.module.Cache().load()
    pyaas.module.Storage().load()



#!/usr/bin/env python

import os

from distutils.core import setup


def add_data_files(*include_dirs):
    data_files = []
    for include_dir in include_dirs:
        for root, directories, filenames in os.walk(include_dir):
            include_files = []
            for filename in filenames:
                include_files.append(os.path.join(root, filename))
            if include_files:
                data_files.append((root, include_files))
    return data_files


setup(
    name='example',
    author='Your Name Here',
    author_email='your@email',
    long_description=open('README.rst').read(),
    version='0.1',
    description='Example Project',
    license='TBD',
    packages=[
        'example',
        'example.handlers',
    ],
    scripts=[
        'bin/example.py',
    ],
    data_files=add_data_files('etc', 'share', 'var'),
    install_requires=[
        'pyaas >= 0.3.9',
    ],
    zip_safe=False
)




VERSION = '0.5.0'

#import sys
#import os
#print '>' * 80
#print __file__
#print sys.path
#print '>' * 80

# a generic error class for throwing exceptions
class error(Exception):
    def __init__(self, fmt, *args):
        self.str = fmt % args

    def __str__(self):
        return self.str


from . import util
from . import settings
from . import module

from .settings import init

prefix      = None
namespace   = None
paths       = None
db          = None
cache       = None



#!/usr/bin/env python

import sys
import os
import atexit
import signal
import time
import logging

import pyaas

pyaas.argparser.add_argument('daemon',
    metavar='(start|stop|restart)',
    help='Control the state of the service'
    )

class Daemonize(object):
    STDIN  = os.path.devnull
    STDOUT = os.path.devnull
    STDERR = os.path.devnull

    def __init__(self, entry, *args, **kwds):
        self.entry = entry
        self.args  = args
        self.kwds  = kwds

        self.pidfile = '/tmp/pyaas-{}.pid'.format(entry.func_name)

        if 'start' == pyaas.args.daemon:
            self.start()
        elif 'stop' == pyaas.args.daemon:
            self.stop()
        elif 'restart' == pyaas.args.daemon:
            self.restart()
        else:
            raise pyaas.error('Unknown daemon option')

    def daemonize(self):
        if not pyaas.args.debug:
            try:
                pid = os.fork()
            except OSError as e:
                logging.critical('EXCEPTION: os.fork: %d (%s)', e.errno, e.strerror)
                sys.exit(-1)

            if pid > 0:
                sys.exit(0)

            # clear environment
            os.chdir("/")
            os.setsid()
            os.umask(0)

            # fork again
            try:
                pid = os.fork()
            except OSError as e:
                logging.critical('EXCEPTION: os.fork: %d (%s)', e.errno, e.strerror)
                sys.exit(-1)

            if pid > 0:
                sys.exit(0)

            # redirect file handles
            sys.stdout.flush()
            sys.stderr.flush()

            stdin  = open(self.STDIN,  'r'    )
            stdout = open(self.STDOUT, 'a+'   )
            stderr = open(self.STDERR, 'a+', 0)

            os.dup2(stdin.fileno(),  sys.stdin.fileno() )
            os.dup2(stdout.fileno(), sys.stdout.fileno())
            os.dup2(stderr.fileno(), sys.stderr.fileno())

        # write pidfile
        #atexit.register(self.delpid)
        def atexit_callback():
            try:
                os.remove(self.pidfile)
            except Exception as e:
                logging.warn('Unable to remove PID file: %s', self.pidfile)

        def signal_callback(signum, frame):
            if signum == signal.SIGTERM:
                sys.exit(0)

        atexit.register(atexit_callback)

        signal.signal(signal.SIGTERM, signal_callback)

        with open(self.pidfile, 'w') as fp:
            fp.write('{}'.format(os.getpid()))

    def _getpid(self):
        pid = None

        if os.path.isfile(self.pidfile):
            try:
                pid = open(self.pidfile, 'r')
                pid = int(pid.read())
            except IOError:
                # TODO: complain not being able to read file
                pid = None
            except ValueError:
                # TODO: complain about invalid value
                pid = None

        return pid

    def start(self):
        pid = self._getpid()

        if pid:
            logging.error('Daemon appears to be running')
            sys.exit(-1)

        # Start the daemon
        self.daemonize()
        self.entry(*self.args, **self.kwds)

    def stop(self):
        pid = self._getpid()

        if not pid:
            logging.warn('Daemon appears to not be running')
            return

        try:
            os.kill(pid, signal.SIGTERM)
        except OSError as e:
            if e.errno != 3:
                logging.warn('Error stopping process: %s', e)
                return

        count = 0
        while count < 100:
            try:
                os.kill(pid, 0)
                time.sleep(0.1)
                count += 1
            except OSError as e:
                if e.errno != 3:
                    logging.warn('Error stopping process: %s', e)
                    return
                else:
                    break

        if count == 100:
            logging.warn('Timeout stopping process')
            return

        if os.path.exists(self.pidfile):
            os.remove(self.pidfile)

    def restart(self):
        self.stop()
        self.start()
